package de.donmanfred;

import static com.sticker4w.StickerContentProvider.ANDROID_APP_DOWNLOAD_LINK_IN_QUERY;
import static com.sticker4w.StickerContentProvider.IOS_APP_DOWNLOAD_LINK_IN_QUERY;
import static com.sticker4w.StickerContentProvider.LICENSE_AGREENMENT_WEBSITE;
import static com.sticker4w.StickerContentProvider.PRIVACY_POLICY_WEBSITE;
import static com.sticker4w.StickerContentProvider.PUBLISHER_EMAIL;
import static com.sticker4w.StickerContentProvider.PUBLISHER_WEBSITE;
import static com.sticker4w.StickerContentProvider.STICKER_FILE_EMOJI_IN_QUERY;
import static com.sticker4w.StickerContentProvider.STICKER_FILE_NAME_IN_QUERY;
import static com.sticker4w.StickerContentProvider.STICKER_PACK_ICON_IN_QUERY;
import static com.sticker4w.StickerContentProvider.STICKER_PACK_IDENTIFIER_IN_QUERY;
import static com.sticker4w.StickerContentProvider.STICKER_PACK_NAME_IN_QUERY;
import static com.sticker4w.StickerContentProvider.STICKER_PACK_PUBLISHER_IN_QUERY;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

import com.sticker4w.Sticker;
import com.sticker4w.StickerContentProvider;
import com.sticker4w.StickerPack;

import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.support.annotation.NonNull;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;

@ShortName("StickerProvider")
//@ActivityObject

//@Permissions(values={"android.permission.INTERNET", "android.permission.ACCESS_NETWORK_STATE"})
//@Events(values={"onSigned(sign As Object)"})
//@DependsOn(values={"android-viewbadger"})
public class StickerContentProviderwrapper {
	private BA ba;
	private String eventName;
	private StickerContentProvider prov; 
	private String Authority;
	/**
	 *  <link>...|http://www.b4x.com</link>
	 */
	public static void LIBRARY_DOC() {
	}

	public void Initialize(BA ba, String EventName, String Authority) {
		this.eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		//StickerContentProvider.CONTENT_PROVIDER_AUTHORITY = Authority;
		this.prov = new StickerContentProvider();
		//StickerContentProvider.setAuthorityURI(Authority);
	}

	public List<StickerPack> getStickerPackList(){
		return prov.getStickerPackList();
	}
	

  /**
   * Get the list of sticker packs for the sticker content provider
   */
  @NonNull
  public static ArrayList<StickerPack> fetchStickerPacks(Context context) throws IllegalStateException {
      final Cursor cursor = context.getContentResolver().query(StickerContentProvider.AUTHORITY_URI, null, null, null, null);
      if (cursor == null) {
          throw new IllegalStateException("could not fetch from content provider, " + StickerContentProvider.CONTENT_PROVIDER_AUTHORITY);
      }
      HashSet<String> identifierSet = new HashSet<>();
      final ArrayList<StickerPack> stickerPackList = fetchFromContentProvider(cursor);
      for (StickerPack stickerPack : stickerPackList) {
          if (identifierSet.contains(stickerPack.identifier)) {
              throw new IllegalStateException("sticker pack identifiers should be unique, there are more than one pack with identifier:" + stickerPack.identifier);
          } else {
              identifierSet.add(stickerPack.identifier);
          }
      }
      if (stickerPackList.isEmpty()) {
          throw new IllegalStateException("There should be at least one sticker pack in the app");
      }
      for (StickerPack stickerPack : stickerPackList) {
          final List<Sticker> stickers = getStickersForPack(context, stickerPack);
          stickerPack.setStickers(stickers);
          //StickerPackValidator.verifyStickerPackValidity(context, stickerPack);
      }
      return stickerPackList;
  }

  @NonNull
  private static List<Sticker> getStickersForPack(Context context, StickerPack stickerPack) {
      final List<Sticker> stickers = fetchFromContentProviderForStickers(stickerPack.identifier, context.getContentResolver());
      for (Sticker sticker : stickers) {
          final byte[] bytes;
          try {
              bytes = fetchStickerAsset(stickerPack.identifier, sticker.imageFileName, context.getContentResolver());
              if (bytes.length <= 0) {
                  throw new IllegalStateException("Asset file is empty, pack: " + stickerPack.name + ", sticker: " + sticker.imageFileName);
              }
              sticker.setSize(bytes.length);
          } catch (IOException | IllegalArgumentException e) {
              throw new IllegalStateException("Asset file doesn't exist. pack: " + stickerPack.name + ", sticker: " + sticker.imageFileName, e);
          }
      }
      return stickers;
  }


  @NonNull
  private static ArrayList<StickerPack> fetchFromContentProvider(Cursor cursor) {
      ArrayList<StickerPack> stickerPackList = new ArrayList<>();
      cursor.moveToFirst();
      do {
          final String identifier = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_PACK_IDENTIFIER_IN_QUERY));
          final String name = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_PACK_NAME_IN_QUERY));
          final String publisher = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_PACK_PUBLISHER_IN_QUERY));
          final String trayImage = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_PACK_ICON_IN_QUERY));
          final String androidPlayStoreLink = cursor.getString(cursor.getColumnIndexOrThrow(ANDROID_APP_DOWNLOAD_LINK_IN_QUERY));
          final String iosAppLink = cursor.getString(cursor.getColumnIndexOrThrow(IOS_APP_DOWNLOAD_LINK_IN_QUERY));
          final String publisherEmail = cursor.getString(cursor.getColumnIndexOrThrow(PUBLISHER_EMAIL));
          final String publisherWebsite = cursor.getString(cursor.getColumnIndexOrThrow(PUBLISHER_WEBSITE));
          final String privacyPolicyWebsite = cursor.getString(cursor.getColumnIndexOrThrow(PRIVACY_POLICY_WEBSITE));
          final String licenseAgreementWebsite = cursor.getString(cursor.getColumnIndexOrThrow(LICENSE_AGREENMENT_WEBSITE));
          final StickerPack stickerPack = new StickerPack(identifier, name, publisher, trayImage, publisherEmail, publisherWebsite, privacyPolicyWebsite, licenseAgreementWebsite);
          stickerPack.setAndroidPlayStoreLink(androidPlayStoreLink);
          stickerPack.setIosAppStoreLink(iosAppLink);
          stickerPackList.add(stickerPack);
      } while (cursor.moveToNext());
      return stickerPackList;
  }

  @NonNull
  private static List<Sticker> fetchFromContentProviderForStickers(String identifier, ContentResolver contentResolver) {
      Uri uri = getStickerListUri(identifier);

      final String[] projection = {STICKER_FILE_NAME_IN_QUERY, STICKER_FILE_EMOJI_IN_QUERY};
      final Cursor cursor = contentResolver.query(uri, projection, null, null, null);
      List<Sticker> stickers = new ArrayList<>();
      if (cursor != null && cursor.getCount() > 0) {
          cursor.moveToFirst();
          do {
              final String name = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_FILE_NAME_IN_QUERY));
              final String emojisConcatenated = cursor.getString(cursor.getColumnIndexOrThrow(STICKER_FILE_EMOJI_IN_QUERY));
              stickers.add(new Sticker(name, Arrays.asList(emojisConcatenated.split(","))));
          } while (cursor.moveToNext());
      }
      if (cursor != null) {
          cursor.close();
      }
      return stickers;
  }

  public static byte[] fetchStickerAsset(@NonNull final String identifier, @NonNull final String name, ContentResolver contentResolver) throws IOException {
      try (final InputStream inputStream = contentResolver.openInputStream(getStickerAssetUri(identifier, name));
           final ByteArrayOutputStream buffer = new ByteArrayOutputStream()) {
          if (inputStream == null) {
              throw new IOException("cannot read sticker asset:" + identifier + "/" + name);
          }
          int read;
          byte[] data = new byte[16384];

          while ((read = inputStream.read(data, 0, data.length)) != -1) {
              buffer.write(data, 0, read);
          }
          return buffer.toByteArray();
      }
  }

  private static Uri getStickerListUri(String identifier) {
  	return new Uri.Builder().scheme(ContentResolver.SCHEME_CONTENT).authority(StickerContentProvider.CONTENT_PROVIDER_AUTHORITY).appendPath(StickerContentProvider.STICKERS).appendPath(identifier).build();
  }

  public static Uri getStickerAssetUri(String identifier, String stickerName) {
  	return new Uri.Builder().scheme(ContentResolver.SCHEME_CONTENT).authority(StickerContentProvider.CONTENT_PROVIDER_AUTHORITY).appendPath(StickerContentProvider.STICKERS_ASSET).appendPath(identifier).appendPath(stickerName).build();
  }

	public String getAuthority() {
		return Authority;
	}

	public void setAuthority(String authority) {
		Authority = authority;
	}
	
}
